aboutsummaryrefslogtreecommitdiffstats
diff options
context:
space:
mode:
authorNick Johnson <arachnid@notdot.net>2017-06-02 01:42:20 +0800
committerNick Johnson <arachnid@notdot.net>2017-06-02 02:14:05 +0800
commitd5a79934dc0542bc3ed74f7561651eecdf62a3a1 (patch)
tree741301566147d659b71bc02d0148b4bfde951b84
parent0424192e61f980272da361f2ed711492b5f65ae0 (diff)
downloaddexon-d5a79934dc0542bc3ed74f7561651eecdf62a3a1.tar
dexon-d5a79934dc0542bc3ed74f7561651eecdf62a3a1.tar.gz
dexon-d5a79934dc0542bc3ed74f7561651eecdf62a3a1.tar.bz2
dexon-d5a79934dc0542bc3ed74f7561651eecdf62a3a1.tar.lz
dexon-d5a79934dc0542bc3ed74f7561651eecdf62a3a1.tar.xz
dexon-d5a79934dc0542bc3ed74f7561651eecdf62a3a1.tar.zst
dexon-d5a79934dc0542bc3ed74f7561651eecdf62a3a1.zip
core/vm: Use a bitmap instead of a map for jumpdest analysis
t push --force
-rw-r--r--core/vm/analysis.go26
1 files changed, 12 insertions, 14 deletions
diff --git a/core/vm/analysis.go b/core/vm/analysis.go
index a0f615821..d5f048d1d 100644
--- a/core/vm/analysis.go
+++ b/core/vm/analysis.go
@@ -22,41 +22,39 @@ import (
"github.com/ethereum/go-ethereum/common"
)
-var bigMaxUint64 = new(big.Int).SetUint64(^uint64(0))
-
// destinations stores one map per contract (keyed by hash of code).
// The maps contain an entry for each location of a JUMPDEST
// instruction.
-type destinations map[common.Hash]map[uint64]struct{}
+type destinations map[common.Hash][]byte
// has checks whether code has a JUMPDEST at dest.
func (d destinations) has(codehash common.Hash, code []byte, dest *big.Int) bool {
// PC cannot go beyond len(code) and certainly can't be bigger than 64bits.
// Don't bother checking for JUMPDEST in that case.
- if dest.Cmp(bigMaxUint64) > 0 {
+ udest := dest.Uint64()
+ if dest.BitLen() >= 63 || udest >= uint64(len(code)) {
return false
}
+
m, analysed := d[codehash]
if !analysed {
m = jumpdests(code)
d[codehash] = m
}
- _, ok := m[dest.Uint64()]
- return ok
+ return (m[udest/8] & (1 << (udest % 8))) != 0
}
// jumpdests creates a map that contains an entry for each
// PC location that is a JUMPDEST instruction.
-func jumpdests(code []byte) map[uint64]struct{} {
- m := make(map[uint64]struct{})
+func jumpdests(code []byte) []byte {
+ m := make([]byte, len(code)/8+1)
for pc := uint64(0); pc < uint64(len(code)); pc++ {
- var op OpCode = OpCode(code[pc])
- switch op {
- case PUSH1, PUSH2, PUSH3, PUSH4, PUSH5, PUSH6, PUSH7, PUSH8, PUSH9, PUSH10, PUSH11, PUSH12, PUSH13, PUSH14, PUSH15, PUSH16, PUSH17, PUSH18, PUSH19, PUSH20, PUSH21, PUSH22, PUSH23, PUSH24, PUSH25, PUSH26, PUSH27, PUSH28, PUSH29, PUSH30, PUSH31, PUSH32:
+ op := OpCode(code[pc])
+ if op == JUMPDEST {
+ m[pc/8] |= 1 << (pc % 8)
+ } else if op >= PUSH1 && op <= PUSH32 {
a := uint64(op) - uint64(PUSH1) + 1
pc += a
- case JUMPDEST:
- m[pc] = struct{}{}
}
}
return m
@@ -22,41 +22,39 @@ import (
// has checks whether code has a JUMPDEST at dest.
func (d destinations) has(codehash common.Hash, code []byte, dest *big.Int) bool {
- // PC cannot go beyond len(code) and certainly can't be bigger than 64bits.
+ // PC cannot go beyond len(code) and certainly can't be bigger than 63bits.
// Don't bother checking for JUMPDEST in that case.